@@ -5,7 +5,7 @@
 # Copyright 1998-1999 - Matt Gerassimoff
 # and Ken Cox <kenstir@senteinc.com>
 #
-# $Id: gnatsweb.pl,v 2.1 1999/04/11 17:11:24 kenstir Exp $
+# $Id: gnatsweb.pl,v 2.2 1999/04/16 22:59:49 kenstir Exp $
 #
 
 #-----------------------------------------------------------------------------
@@ -26,11 +26,26 @@
 $site_gnats_host = 'localhost';
 $site_gnats_port = 1529;
 
+# Set to true if you compiled gnats with GNATS_RELEASE_BASED defined.
+$site_release_based = 0;
+
 # Name you want in the page banner.
 $site_banner_text = 'gnatsweb';
 
 # Program to send email notifications.
-$site_mailer = '/usr/sbin/sendmail -oi -t';
+if (-x '/usr/sbin/sendmail')
+{
+  $site_mailer = '/usr/sbin/sendmail -oi -t';
+}
+elsif (-x '/usr/lib/sendmail')
+{
+  $site_mailer = '/usr/lib/sendmail -oi -t';
+}
+else
+{
+  die("Can't locate 'sendmail'; must set \$site_mailer in gnats-site.pl");
+}
+
 
 # site_callback -
 #
@@ -70,7 +85,7 @@ use IO::Handle;
 use POSIX;
 
 # get RCS tag as just a number
-($REVISION  = '$Revision: 2.1 $ ') =~ s/.Revision: (.*) ../$1/;
+($REVISION  = '$Revision: 2.2 $ ') =~ s/.Revision: (.*) ../$1/;
 
 # width of text fields
 $textwidth = 60;
@@ -334,13 +349,15 @@ sub sendpr
   shift(@priority);
   shift(@class);
   shift(@confidential);
-  $q->delete('Submitter-Id');
-  my $default_email = $email || $user;
+  shift(@responsible);
+  shift(@state);
+  shift(@submitter_id);
+
   print $q->start_form(),
 	"<table>",
 	"<tr><td><b>E-Mail Address:</b><td>",
 	$q->textfield(-name=>'email',
-	              -default=>$default_email,
+	              -default=>$email,
 		      -size=>$textwidth),
 	"<tr><td><b>CC:</b><td>",
 	$q->textfield(-name=>'cc',
@@ -349,13 +366,15 @@ sub sendpr
   foreach (@fieldnames)
   {
     next if ($fieldnames{$_} & $SENDEXCLUDE);
-    my $lc_fieldname = lc $_;
+    my $lc_fieldname = field2param($_);
 
     # Get default value from site_callback if provided, otherwise take
     # our defaults.
     my $default;
-    $default = 'serious' if /Severity/;
-    $default = 'medium'  if /Priority/;
+    $default = 'serious'                  if /Severity/;
+    $default = 'medium'                   if /Priority/;
+    $default = $submitter_id || 'unknown' if /Submitter-Id/;
+    $default = $originator                if /Originator/;
     $default = cb("sendpr_$lc_fieldname") || $default;
 
     if ($fieldnames{$_} & $ENUM)
@@ -395,6 +414,7 @@ sub submitnewpr
 {
   my $page = 'Create PR Results';
   page_start_html($page);
+  my $debug = 0;
 
   my(@values, $key);
   my(%fields);
@@ -415,15 +435,23 @@ sub submitnewpr
     print "You must fill in the E-Mail Address field.\n";
     return;
   }
-  $fields{'Originator'} = $fields{'email'};
+  $fields{'Originator'} = $fields{'Originator'} || $fields{'email'};
   $text = unparsepr("send", %fields);
+
+  if($debug)
+  {
+    print "<h3>debugging</h3>",
+          $q->pre($q->escapeHTML($text));
+    page_end_html($page);
+    return;
+  }
+
   if(!open(MAIL, "|$site_mailer"))
   {
     page_heading($page, 'Error');
     print "<h3>Error invoking $site_mailer</h3>";
     return;
   }
-  #my $to = 'kenstir@senteinc.com';
   my $to = $GNATS_ADDR;
   $text = <<EOT . $text;
 To: $to
@@ -436,8 +464,6 @@ X-gnatsweb-version: $REVISION
 EOT
 
   print MAIL $text;
-  #print "<h3>Report that was sent:</h3><br>";
-  #print $q->pre(CGI::escapeHTML($text));
   if(!close(MAIL))
   {
     page_heading($page, 'Error');
@@ -476,12 +502,13 @@ sub view
   local(%fields) = readpr($pr);
 
   print $q->start_form(),
-	$q->hidden('pr', $pr),
-        $q->submit('cmd', 'edit');
-  if(!$viewaudit)
-  {
-    print " or ", $q->submit('cmd', 'view audit-trail');
-  }
+	$q->hidden('pr', $pr);
+
+  # print 'edit' and 'view audit-trail' buttons as appropriate
+  print $q->submit('cmd', 'edit')             if (can_edit());
+  print " or "                                if (can_edit() && !$viewaudit);
+  print $q->submit('cmd', 'view audit-trail') if (!$viewaudit);
+
   print $q->hr(),
         "<table>";
   foreach (@fieldnames)
@@ -499,12 +526,13 @@ sub view
           $q->tt($val), "\n";
   }
   print "</table>",
-        $q->hr(),
-        $q->submit('cmd', 'edit');
-  if(!$viewaudit)
-  {
-    print " or ", $q->submit('cmd', 'view audit-trail');
-  }
+        $q->hr();
+
+  # print 'edit' and 'view audit-trail' buttons as appropriate
+  print $q->submit('cmd', 'edit')             if (can_edit());
+  print " or "                                if (can_edit() && !$viewaudit);
+  print $q->submit('cmd', 'view audit-trail') if (!$viewaudit);
+
   print $q->end_form();
 
   # Footer comes before the audit-trail.
@@ -546,12 +574,20 @@ sub edit
 #    print "</pre></font><hr>\n";
 #  }
 
+  # 4/12/99 kenstir: "Carl E. Lindberg" <lindberg@BLaCKSMITH.com> says
+  # that his old (gnats 3.2) PR's contain email addrs in the Responsible
+  # field.  This doesn't change anything for my PR's.
+  $fields{'Responsible'} = fix_email_addr($fields{'Responsible'});
+
   # remove "all" from arrays
   shift(@category);
   shift(@severity);
   shift(@priority);
   shift(@class);
   shift(@confidential);
+  shift(@responsible);
+  shift(@state);
+  shift(@submitter_id);
 
   print $q->start_form(),
         $q->submit('cmd', 'submit edit'),
@@ -568,7 +604,7 @@ sub edit
   foreach (@fieldnames)
   {
     next if ($fieldnames{$_} & $EDITEXCLUDE);
-    my $lc_fieldname = lc $_;
+    my $lc_fieldname = field2param($_);
 
     if ($fieldnames{$_} & $ENUM)
     {
@@ -629,7 +665,7 @@ sub submitedit
   if(!$pr)
   {
     page_heading($page, 'Error');
-    print "<h3>ERROR: You must specify a problem report number</h3>";
+    print "<h3>You must specify a problem report number</h3>";
     return;
   }
 
@@ -717,7 +753,9 @@ sub submitedit
 	    if($fields{"$_-Why"} =~ /^\s*$/)
 	    {
               page_heading($page, 'Error') if $ok;
-	      print "<h3>Field '$_' must have a reason for change</h3>";
+	      print "<h3>Field '$_' must have a reason for change</h3>",
+                    "Old $_: $oldfields{$_}<br>",
+                    "New $_: $fields{$_}";
 	      $ok = 0;
 	    }
             else
@@ -746,18 +784,19 @@ sub submitedit
 	if(!defined($adr = praddr($fields{'Responsible'})))
 	{
           page_heading($page, 'Error');
-	  print "<h3>Cannot find address for $fields{'Responsible'}.</h3>";
+	  print "<h3>Cannot get email address for $fields{'Responsible'}</h3>",
+                "New Responsible: $fields{'Responsible'}";
 	  last LOCKED;
 	}
 	$mailto{lc($adr)} = 1;
       }
-      if(!defined($adr = praddr($oldfields{'Responsible'})))
+      # 4/12/99 kenstir: Don't warn if you can't get an email address
+      # for the old Responsible party.  It may have been invalid, or
+      # they may have left the company.
+      if(defined($adr = praddr($oldfields{'Responsible'})))
       {
-        page_heading($page, 'Error');
-	print "<h3>Cannot find address for $oldfields{'Responsible'}.</h3>";
-	last LOCKED;
+        $mailto{lc($adr)} = 1;
       }
-      $mailto{lc($adr)} = 1;
       $fields{'Audit-Trail'} = $oldfields{'Audit-Trail'} . $audittrail;
       my($newpr) = unparsepr("", %fields);
       $newpr =~ s/\r//g;
@@ -831,6 +870,8 @@ sub query_page
   page_heading($page, 'Query Problem Reports', 1);
   print_stored_queries();
   print $q->start_form(),
+	$q->submit('cmd', 'submit query'),
+        "<hr>",
 	"<table>",
 	"<tr><td>Category:<td>",
 	$q->popup_menu(-name=>'category',
@@ -868,15 +909,15 @@ sub query_page
 	$q->checkbox_group(-name=>'originatedbyme',
 	               -values=>['Originated by You'],
 		       -defaults=>[]),
-	"<tr><td>Column Display:<td>",
-	"</table>",
-	"<table>";
+	"<tr valign=top><td>Column Display:<td>";
   my $defaultsref = @columns ? \@columns : \@deffields;
-  print $q->checkbox_group(-name=>'columns',
+  print $q->scrolling_list(-name=>'columns',
                            -values=>\@fields,
-                           -defaults=>$defaultsref),
+                           -defaults=>$defaultsref,
+                           -multiple=>1,
+                           -size=>5),
 	"</table>",
-        "<br><br>",
+        "<hr>",
 	$q->submit('cmd', 'submit query'),
         $q->end_form();
 
@@ -896,7 +937,9 @@ sub advanced_query_page
   my $heading_bg = '#9fbdf9';
   my $cell_bg = '#d0d0d0';
 
-  print $q->submit('cmd', 'submit query');
+  print $q->submit('cmd', 'submit query'),
+	" or ",
+	$q->reset(-name=>'reset');
   print "<hr>";
   print "<center>";
 
@@ -924,9 +967,12 @@ sub advanced_query_page
         "<th nowrap>Date Search",
         "<th nowrap>Example: <tt>1999-04-01 05:00 GMT</tt>",
         "</tr>\n";
-  foreach ('Arrived Before', 'Arrived After',
-           'Modified Before', 'Modified After',
-           'Closed Before', 'Closed After')
+  my(@date_queries) =  ('Arrived After', 'Arrived Before',
+                        'Modified After', 'Modified Before',
+                        'Closed After', 'Closed Before');
+  push(@date_queries, 'Required After', 'Required Before')
+        if $site_release_based;
+  foreach (@date_queries)
   {
     my $param_name = lc($_);
     $param_name =~ s/ //;
@@ -947,7 +993,7 @@ sub advanced_query_page
         "</tr>\n";
   foreach (@fieldnames)
   {
-    my $lc_fieldname = lc $_;
+    my $lc_fieldname = field2param($_);
     next unless ($gnatsd_query{$lc_fieldname});
 
     print "<tr valign=top>";
@@ -1004,6 +1050,8 @@ sub advanced_query_page
   print "</center>\n";
   print "<hr>",
 	$q->submit('cmd', 'submit query'),
+	" or ",
+	$q->reset(-name=>'reset'),
 	$q->end_form();
   page_footer($page);
   page_end_html($page);
@@ -1066,6 +1114,10 @@ sub by_field
   {
     $val = $a->[1] cmp $b->[1];
   }
+  elsif ($sortby eq 'Confidential')
+  {
+    $val = $a->[3] cmp $b->[3];
+  }
   elsif ($sortby eq 'Severity')
   {
     # sort by Severity then Priority then Class
@@ -1098,10 +1150,14 @@ sub by_field
   {
     $val = $a->[8] <=> $b->[8];
   }
-  elsif ($sortby eq 'Release')
+  elsif ($sortby eq 'Submitter-Id')
   {
     $val = $a->[9] cmp $b->[9];
   }
+  elsif ($sortby eq 'Release')
+  {
+    $val = $a->[12] cmp $b->[12];
+  }
   else
   {
     $val = $a->[0] <=> $b->[0];
@@ -1109,6 +1165,42 @@ sub by_field
   $val;
 }
 
+# nonempty -
+#     Turn empty strings into "&nbsp;" so that Netscape tables won't
+#     look funny.
+#
+sub nonempty
+{
+  my $str = shift;
+  $str = '&nbsp;' if !$str;
+  $str;
+}
+
+# field2param -
+#     Convert gnats field name into parameter name, e.g.
+#     "Submitter-Id" => "submitter_id".  It's done this crazy way for
+#     compatibility with queries stored by gnatsweb 2.1.
+#
+sub field2param
+{
+  my $name = shift;
+  $name =~ s/-/_/g;
+  $name = lc($name);
+}
+
+# param2field -
+#     Convert parameter name into gnats field name, e.g.
+#     "submitter_id" => "Submitter-Id".  It's done this crazy way for
+#     compatibility with queries stored by gnatsweb 2.1.
+#
+sub param2field
+{
+  my $name = shift;
+  my @words = split(/_/, $name);
+  map { $_ = ucfirst($_); } @words;
+  $name = join('-', @words);
+}
+
 # display_query_results -
 #     Display the query results, and the "store query" form.
 sub display_query_results
@@ -1128,28 +1220,12 @@ sub display_query_results
   # split() only once per item, as opposed to during every comparison.
   # Note that $sortby must be 'local'...it's used in by_field().
   local($sortby) = $q->param('sortby');
-  my(@sortable) = ('PR','Category','Severity','Priority','Responsible',
-                   'State','Class','Release');
+  my(@sortable) = ('PR','Category','Confidential',
+                   'Severity','Priority','Responsible',
+                   'State','Class','Release','Submitter-Id');
   my(@presplit_prs) = map { [ (split /\|/) ] } @query_results;
   my(@sorted_prs) = sort by_field @presplit_prs;
 
-#  print '<hr><h2>query_results</h2><pre>';
-#  foreach (@query_results) {
-#    print "$_\n";
-#  }
-#  print '</pre><hr><h2>presplit_prs</h2><pre>';
-#  foreach (@presplit_prs) {
-#    print "$_->[0]\t", join('/', @{$_}), "\n";
-#  }
-#  print '</pre><hr><h2>sorted_prs</h2><pre>';
-#  foreach (@sorted_prs) {
-#    print "$_->[0]\t", join('/', @{$_}), "\n";
-#    my($id, $cat) = @{$_};
-#    print "$_->[0]\t$id\t$cat\n";
-#  }
-#  print '</pre><hr>';
-#  exit;
-
   print $q->start_form(),
 	$q->hidden(name=>'cmd', -value=>'view', -override=>1),
 	"<table border=1>";
@@ -1160,8 +1236,7 @@ sub display_query_results
   print "<tr>";
   for $field ('PR', @fields)
   {
-    # @fields is in lower case, @sortable in initial cap
-    $ufield = ucfirst $field;
+    $ufield = param2field($field);
     if (grep(/$ufield/, @sortable))
     {
       $q->param(-name=>'sortby', -value=>$ufield);
@@ -1178,23 +1253,36 @@ sub display_query_results
   $q->param(-name=>'sortby', -value=>$sortby);
   print "</tr>";
 
-  foreach $_ (@sorted_prs)
+  # Print the PR's.
+  foreach (@sorted_prs)
   {
     print "<tr valign=top>";
+
     my($id, $cat, $syn, $conf, $sev,
        $pri, $resp, $state, $class, $sub,
-       $arrival, $orig, $release, $lastmoddate, $closeddate)
-          = @{$_};
+       $arrival, $orig, $release, $lastmoddate, $closeddate,
+       $target, $keywords, $daterequired) = @{$_};
     print "<td nowrap><a href=\"$sn?cmd=view&pr=$id\">$id</a>"; 
-    print "<td nowrap>$cat"               if $fields{'category'};
-    print "<td nowrap>" . $state[$state]  if $fields{'state'};
-    print "<td nowrap>" . $class[$class]  if $fields{'class'};
-    print "<td nowrap>" . $severity[$sev] if $fields{'severity'};
-    print "<td nowrap>" . $priority[$pri] if $fields{'priority'};
-    print "<td nowrap>$resp"              if $fields{'responsible'};
-    print "<td nowrap>$sub"               if $fields{'submitter'};
-    print "<td nowrap>$orig"              if $fields{'originator'};
-    print "<td>$syn"                      if $fields{'synopsis'};
+    print "<td nowrap>$cat"                      if $fields{'category'};
+    print "<td nowrap>$conf"                     if $fields{'confidential'};
+    print "<td nowrap>$state[$state]"            if $fields{'state'};
+    print "<td nowrap>$class[$class]"            if $fields{'class'};
+    print "<td nowrap>$severity[$sev]"           if $fields{'severity'};
+    print "<td nowrap>$priority[$pri]"           if $fields{'priority'};
+    print "<td nowrap>", nonempty($release)      if $fields{'release'};
+    print "<td nowrap>", nonempty($target)       if($site_release_based
+                                                    && $fields{'target'});
+    print "<td nowrap>", nonempty($keywords)     if($site_release_based
+                                                    && $fields{'keywords'});
+    print "<td nowrap>$resp"                     if $fields{'responsible'};
+    print "<td nowrap>", nonempty($sub)          if $fields{'submitter_id'};
+    print "<td nowrap>", nonempty($orig)         if $fields{'originator'};
+    print "<td nowrap>$arrival"                  if $fields{'arrival_date'};
+    print "<td nowrap>", nonempty($daterequired) if($site_release_based
+                                                    && $fields{'date_required'});
+    print "<td nowrap>", nonempty($lastmoddate)  if $fields{'last_modified'};
+    print "<td nowrap>", nonempty($closeddate)   if $fields{'closed_date'};
+    print "<td>$syn"                             if $fields{'synopsis'};
     print "</tr>\n";
   }
   print "</table>",
@@ -1358,9 +1446,9 @@ sub one_line_form
 {
   my($label, @form_body) = @_;
   my $valign = 'baseline';
-  print Tr({-valign=>$valign},
-           td(b($label)),
-           td(start_form(), @form_body, end_form()));
+  print $q->Tr({-valign=>$valign},
+               $q->td($q->b($label)),
+               $q->td(start_form(), @form_body, end_form()));
 }
 
 # can_edit -
@@ -1467,8 +1555,8 @@ sub page_start_html
     return;
   }
 
-  print start_html(-title=>"$title - $site_banner_text",
-                   -bgcolor=>'#ffffff');
+  print $q->start_html(-title=>"$title - $site_banner_text",
+                       -bgcolor=>'#ffffff');
 
   # Add the page banner.  This banner is a string slammed to the right
   # of a 100% width table.  The data is a link back to the main page.
@@ -1507,12 +1595,12 @@ END_OF_STYLE
 END_OF_STYLE
   }
   my($row, $banner);
-  $row = Tr(td({-align=>'right'},
-               a({-style=>$style, -href=>$sn},
-                 ' ', $site_banner_text, ' ')));
-  $banner = table({-bgcolor=>'#000000', -width=>'100%',
-                   -border=>0, -cellpadding=>0, -cellspacing=>0},
-                  $row);
+  $row = $q->Tr($q->td({-align=>'right'},
+                       $q->a({-style=>$style, -href=>$sn},
+                             ' ', $site_banner_text, ' ')));
+  $banner = $q->table({-bgcolor=>'#000000', -width=>'100%',
+                       -border=>0, -cellpadding=>0, -cellspacing=>0},
+                      $row);
   print $banner;
 
   # debugging
@@ -1551,16 +1639,19 @@ sub page_heading
 
   if ($user && defined($display_user_info))
   {
-    $rightcol= tt("User: $user<br>Database: $database");
+    $rightcol= "<tt><small>User: $user<br>" .
+               "Database: $database<br>" .
+               "Access: $access_level</tt></small>";
   }
   else
   {
     $rightcol = '&nbsp;';
   }
 
-  print table(Tr(td({-nowrap=>1}, font({-size=>'+3'}, $leftcol)),
-                 td({-width=>'100%'}, '&nbsp;'), # empty expandable filler
-                 td({-nowrap=>1}, $rightcol)));
+  print $q->table($q->Tr($q->td({-nowrap=>1}, font({-size=>'+3'}, $leftcol)),
+                         # this column serves as empty expandable filler
+                         $q->td({-width=>'100%'}, '&nbsp;'),
+                         $q->td({-nowrap=>1}, $rightcol)));
 }
 
 # page_footer -
@@ -1615,9 +1706,15 @@ sub initialize
   @priority = ("all", "high", "medium", "low");
   @category = ("all");
   @confidential = ("all", "no", "yes");
-  @fields = ("category", "state", "class", "severity", "priority",
-             "responsible", "submitter", "originator", "synopsis");
+
+  # @fields - param names of columns displayable in query results
+  # @deffields - default displayed columns
   @deffields = ("category", "state", "responsible", "synopsis");
+  @fields = ("category", "confidential", "state", "class",
+             "severity", "priority",
+             "release", "target", "responsible", "submitter_id", "originator",
+             "arrival_date", "date_required",
+             "last_modified", "closed_date", "synopsis");
 
   # @fieldnames - fields appear in the standard order, defined by pr.h
   @fieldnames = (
@@ -1629,9 +1726,9 @@ sub initialize
     "Priority",
     "Responsible",
     "State",
-    # "Quarter",
-    # "Keywords",
-    # "Date-Required",
+    "Target",
+    "Keywords",
+    "Date-Required",
     "Class",
     "Submitter-Id",
     "Arrival-Date",
@@ -1659,17 +1756,17 @@ sub initialize
     "Priority"      => $ENUM,
     "Responsible"   => $ENUM | $REASONCHANGE | $SENDEXCLUDE,
     "State"         => $ENUM | $REASONCHANGE | $SENDEXCLUDE,
-    #"Quarter"       => 0,
-    #"Keywords"      => 0,
-    #"Date-Required" => 0,
+    "Target"        => 0,
+    "Keywords"      => 0,
+    "Date-Required" => 0,
     "Class"         => $ENUM,
-    "Submitter-Id"  => $SENDEXCLUDE | $EDITEXCLUDE,
+    "Submitter-Id"  => $ENUM | $EDITEXCLUDE,
     "Arrival-Date"  => $SENDEXCLUDE | $EDITEXCLUDE,
     "Closed-Date"   => $SENDEXCLUDE | $EDITEXCLUDE,
     "Last-Modified" => $SENDEXCLUDE | $EDITEXCLUDE,
-    "Originator"    => $SENDEXCLUDE | $EDITEXCLUDE,
+    "Originator"    => $EDITEXCLUDE,
     "Release"       => 0,
-    "Organization"  => $MULTILINE | $SENDEXCLUDE | $EDITEXCLUDE,
+    "Organization"  => $MULTILINE | $SENDEXCLUDE | $EDITEXCLUDE, # => $MULTILINE
     "Environment"   => $MULTILINE,
     "Description"   => $MULTILINE,
     "How-To-Repeat" => $MULTILINE,
@@ -1678,8 +1775,7 @@ sub initialize
     "Audit-Trail"   => $MULTILINE | $SENDEXCLUDE | $EDITEXCLUDE,
     "Unformatted"   => $MULTILINE | $SENDEXCLUDE | $EDITEXCLUDE,
   );
-  # gnatsd query commands: maps lc(field_name) to gnatsd command
-  # (some keys are not actually field names, but form element names)
+  # gnatsd query commands: maps param name to gnatsd command
   %gnatsd_query = (
     "category"        => 'catg',
     "synopsis"        => 'synp',
@@ -1689,7 +1785,7 @@ sub initialize
     "responsible"     => 'resp',
     "state"           => 'stat',
     "class"           => 'clss',
-    "submitter-id"    => 'subm',
+    "submitter_id"    => 'subm',
     "originator"      => 'orig',
     "release"         => 'rlse',
     "text"            => 'text',
@@ -1700,8 +1796,19 @@ sub initialize
     "modifiedafter"   => 'maft',
     "closedbefore"    => 'cbfr',
     "closedafter"     => 'caft',
+    "target"	      => 'qrtr',
+    "keywords"	      => 'keyw',
+    "requiredbefore"  => 'bfor',
+    "requiredafter"   => 'aftr',
   );
 
+  # clear out some unused fields if not used
+  if (!$site_release_based)
+  {
+    @fields = grep(!/target|keywords|date_required/, @fields);
+    @fieldnames = grep(!/Target|Keywords|Date-Required/, @fieldnames);
+  }
+
   my $response;
 
   # Get gnatsd version from initial server connection text.
@@ -1734,7 +1841,9 @@ sub initialize
     }
   }
 
-  foreach $_ (client_cmd("lcat"))
+  my @tmp;
+  @tmp = client_cmd("lcat");
+  foreach $_ (sort @tmp)
   {
     my($cat, $desc, $resp, $notify) = split(/:/);
     push(@category, $cat);
@@ -1747,7 +1856,8 @@ sub initialize
     push(@state, $x);
   }
   @responsible = ("all");
-  foreach $_ (client_cmd("lres"))
+  @tmp = client_cmd("lres");
+  foreach $_ (sort @tmp)
   {
     ($x, $dummy) = split(/:/);
     push(@responsible, $x);
@@ -1758,6 +1868,35 @@ sub initialize
     ($x, $dummy) = split(/:/);
     push(@class, $x);
   }
+  @submitter_id = ("all");
+  @tmp = client_cmd("lsub");
+  foreach $_ (sort @tmp)
+  {
+    ($x, $dummy) = split(/:/);
+    push(@submitter_id, $x);
+  }
+
+  # Now that everything's all set up, let the site_callback have at it.
+  # It's return value doesn't matter, but here it can muck with our defaults.
+  cb('initialize');
+}
+
+# fix_email_addr -
+#     Trim email addresses as it appears in an email From or Reply-To
+#     header (and sometimes in the Responsible field of old PR's), into
+#     something we can use to call sendmail with.
+#
+#     Delete everything inside parenthesis and outside <>'s, inclusive.
+#
+sub fix_email_addr
+{
+  my $arg = shift;
+  $arg = lc($arg);
+  $arg =~ s/\(.*\)//;
+  $arg =~ s/.*<(.*)>.*/$1/;
+  $arg =~ s/^\s+//;
+  $arg =~ s/\s+$//;
+  $arg;
 }
 
 sub parsepr
@@ -1799,16 +1938,12 @@ sub parsepr
       $fields{$hdrmulti} .= $_;
     }
 
+    # Grab a few fields out of the envelope as it flies by
     if($hdr eq "Reply-To" || $hdr eq "From")
     {
-      # Grab a few fields out of the envelope as it flies by
-      $arg = lc($arg);
-      #  Delete everything inside parenthesis and outside <>'s, inclusive.
-      $arg =~ s/\(.*\)//;
-      $arg =~ s/.*<(.*)>.*/$1/;
-      $arg =~ s/^\s+//;
-      $arg =~ s/\s+$//;
-      print "<h3>error: internal whitespace in Reply-to: or From: header!</h3>" if ($arg =~ /\s/);
+      $arg = fix_email_addr($arg);
+      print "<h3>error: internal whitespace in Reply-to: or From: header!</h3>"
+            if ($arg =~ /\s/);
       $fields{$hdr} = $arg;
       #print "storing, hdr = $hdr, arg = $arg\n";
     }
@@ -1819,6 +1954,9 @@ sub parsepr
   return %fields;
 }
 
+# unparsepr -
+#     Turn PR hash back into a string.
+#     'send' arg is used when creating a new PR only.
 sub unparsepr
 {
   my($send, %fields) = @_;
@@ -1830,8 +1968,12 @@ sub unparsepr
     if($fieldnames{$_} & $MULTILINE)
     {
       # Lines which begin with a '.' need to be escaped by another '.'
+      # unless we're using sendmail to send a new PR.
+      # TODO: When gnatsd supports creating a new PR, then we'll need to
+      # do the escape again.
       $tmp = $fields{$_};
-      $tmp =~ s/^[.]/../gm;
+      $tmp =~ s/^[.]/../gm
+            unless ($send eq 'send');
       $text .= sprintf(">$_:\n%s", $tmp);
     }
     else
@@ -1940,14 +2082,17 @@ sub main
     exit();
   }
 
-  # Retrieve the gnatsweb cookie from the browser.
+  # Retrieve the gnatsweb cookie sent by the browser.
   %cookie = $q->cookie('gnatsweb');
   $user     = $cookie{'user'};
   $password = $cookie{'password'};
   $database = $cookie{'database'};
   $email    = $cookie{'email'};
   $cc       = $cookie{'cc'};
+  $submitter_id = $cookie{'Submitter-Id'};
+  $originator   = $cookie{'Originator'};
   @columns  = split(' ', $cookie{'columns'});
+  #foreach $key (sort keys %cookie) { warn "debug 1: $key = $cookie{$key}\n";}
 
   # Upon login, store user/password/database in the gnatsweb cookie.
   if($cmd eq 'login')
@@ -1962,14 +2107,17 @@ sub main
   # next time.
   if($cmd eq 'submit')
   {
-    $cookie{'email'} = $email = $q->param('email');
-    $cookie{'cc'}    = $cc    = $q->param('cc');
+    $cookie{'email'}          = $email        = $q->param('email');
+    $cookie{'cc'}             = $cc           = $q->param('cc');
+    $cookie{'Submitter-Id'}   = $submitter_id = $q->param('Submitter-Id');
+    $cookie{'Originator'}     = $originator   = $q->param('Originator');
     #$cookie{'Category'}    = $Category    = $q->param('Category');
     #$cookie{'Release'}     = $Release     = $q->param('Release');
     #$cookie{'Environment'} = $Environment = $q->param('Environment');
+    #foreach $key (sort keys %cookie) { warn "debug 2: $key = $cookie{$key}\n";}
   }
 
-  # Upon a 'submit query', store column display list.
+  # Upon a 'submit query', store column display list in the cookie.
   if($cmd eq 'submit query')
   {
     @columns = $q->param('columns');
@@ -1980,7 +2128,8 @@ sub main
   # that it doesn't expire.
   $new_cookie = $q->cookie(-name => 'gnatsweb', 
                            -value => \%cookie,
-                           -expires => '+10y');
+                           -expires => '+1y');
+  #warn "debug 3:\n$new_cookie\n";
 
   # Serve the cookie.
   print $q->header(-cookie=>$new_cookie);
