@@ -3,12 +3,15 @@
 # gnats-test.pl -
 #      Test code for gnats-lib.pl
 #
-# $Id: test.pl,v 2.1 1999/04/11 17:11:25 kenstir Exp $
+# $Id: test.pl,v 2.3 1999/04/16 22:59:50 kenstir Exp $
 
 #-----------------------------------------------------------------------------
 # test harness setup
 
 use POSIX;
+use Getopt::Std;
+
+getopts('d');
 
 $suppress_main = 1;
 do 'gnatsweb.pl';
@@ -18,6 +21,9 @@ $errors = 0;
 
 open(LOG, ">test.log") || die;
 
+# turn on debug options if -d
+$client_cmd_debug = $opt_d;
+
 #-----------------------------------------------------------------------------
 # support subs
 
